import os
import yaml

from jinja2 import Template
from .util import dynamic_import


class ConfigBase(object):

    """
    Base config class
    """

    def is_valid(self):
        return True


class RiemannConfig(ConfigBase):

    """
    Riemann Config object
    """

    def __init__(self, cfg):
        self._data = cfg

    @property
    def host(self):
        return self._data.get("host", "localhost")

    @property
    def port(self):
        return int(self._data.get("port", 5555))

    @staticmethod
    def default():
        return RiemannConfig({})


class AgentConfig(ConfigBase):

    """
    Config for setuping agent
    """

    def __init__(self, cfg):
        self._data = cfg
        self._instance = None

    @property
    def name(self):
        return self._data["name"]

    @property
    def module(self):
        return self._data["module"]

    @property
    def instance(self):
        if self._instance is None:
            self._instance = dynamic_import(self.module)(self._data)
        return self._instance

    @property
    def tag(self):
        return self._data.get("tag", None)

    def is_valid(self):
        return self.instance.is_valid()


class Config(ConfigBase):

    """
    Config object
    """

    def __init__(self, cfg):
        self._data = cfg
        self.is_valid()

    def is_valid(self):
        checks = ([self.riemann.is_valid] +
                  list(map(lambda x: x.is_valid, self.agents)))
        return all([c() for c in checks])

    @property
    def riemann(self):
        if "riemann" in self._data:
            return RiemannConfig(self._data["riemann"])
        else:
            return RiemannConfig.default()

    @property
    def interval(self):
        return self._data["interval"]

    @property
    def agents(self):
        return [AgentConfig(a) for a in self._data.get("agents", [])]


def load(config_file):
    """
    Processes and loads config file.
    """
    with open(config_file, "r") as f:

        def env_get():
            return dict(os.environ)
        tmpl = Template(f.read())
        return Config(yaml.load(tmpl.render(**env_get())))



import aiohttp

from time import time
from . import Agent


class Success(object):

    def __repr__(self):
        return "success"


class Failure(object):

    def __init__(self, code):
        self.code = code

    def __repr__(self):
        return "failure"


class HttpAgent(Agent):

    def is_valid(self):
        checks = [("url" in self._data)
                  ]
        return all(checks)

    @property
    def url(self):
        return self._data["url"]

    @property
    def headers(self):
        return self._data.get("headers", [])

    @property
    def cookies(self):
        return self._data.get("cookies", [])

    def translate_status(self, code):
        if code in [200, 201, 202, 204, 205, 206]:
            return Success()
        else:
            return Failure(code)

    async def process(self, event_fn):
        logger = self.get_logger()
        ts = time()
        state = None
        async with aiohttp.ClientSession() as session:
            async with session.get(self.url) as resp:
                state = self.translate_status(resp.status)
        te = time()
        span = int((te - ts) * 1000)
        logger.debug("Request to {url} returned status code {code} (as {state}) "
                     "in {span} milliseconds.".format(url=self.url,
                                                      code=resp.status,
                                                      state=state,
                                                      span=span))

        event_fn(service=self.prefix + "health",
                 metric_f=span,
                 state=str(state),
                 description=self.url)



import aiohttp

from functools import partial

from . import Agent


class ConsulAgent(Agent):

    @property
    def leader_url(self):
        return "http://{host}:{port}/v1/status/leader".format(host=self.host,
                                                              port=self.port)

    @property
    def services_url(self):
        return ("http://{host}:{port}/v1/catalog/services"
                .format(host=self.host,
                        port=self.port))

    @property
    def nodes_url(self):
        return ("http://{host}:{port}/v1/catalog/nodes"
                .format(host=self.host,
                        port=self.port))

    @property
    def health_url(self):
        def health(host, port, service_name):
            return ("http://{host}:{port}/v1/health/service/{service}"
                    .format(host=host,
                            port=port,
                            service=service_name))

        return partial(health, self.host, self.port)

    async def get_json(self, url):
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as resp:
                return await resp.json()

    @property
    async def services(self):
        return await self.get_json(self.services_url)

    @property
    async def nodes(self):
        return await self.get_json(self.nodes_url)

    async def health(self, service_name):
        return await self.get_json(self.health_url(service_name))

    @property
    def host(self):
        return self._data.get("host", "localhost")

    @property
    def port(self):
        return int(self._data.get("port", 8500))

    async def process(self, event_fn):
        logger = self.get_logger()
        services = await self.services
        logger.debug("Got services: {0}".format(services))

        nodes = await self.nodes
        logger.debug("Got nodes: {0}".format(nodes))
        for service_name, _ in services.items():
            health_nodes = await self.health(service_name)
            for node in health_nodes:
                logger.debug("HealthInfo: {0}".format(node))
                ok = all([check["Status"] == "passing" for check in node["Checks"]])
                logger.debug("Node is Ok? {0}".format(ok))
                event_fn(metric_f=1.0 if ok else 0.0,
                         service=self.prefix + service_name,
                         host=node["Node"]["Node"])



"""
Module just to print out current version.
That's all.
"""

VERSION = (0, 0, 6)


def get_version():
    """
    Returns version
    """
    return ".".join(map(str, VERSION))


if __name__ == '__main__':
    print(get_version())



from logbook import Logger


class Agent(object):

    def __init__(self, cfg):
        self._data = cfg

    @property
    def prefix(self):
        return "{0}.".format(self._data["name"])

    def get_logger(self):
        return Logger(self.__class__.__name__)

    async def process(self, event_fn):
        pass

    def is_valid(self):
        return True



import click
from dotenv import load_dotenv, find_dotenv
from .config import load
from .core.heart import start_loop

load_dotenv(find_dotenv())


@click.command()
@click.option("--config", help="Config file", default="config.yaml")
def main(config):
    cfg = load(config)
    start_loop(cfg)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from setuptools import setup
from pip.req import parse_requirements
from pip.exceptions import InstallationError

from oshino.version import get_version

try:
    install_reqs = list(parse_requirements("requirements.txt", session={}))
except InstallationError:
    # There are no requirements
    install_reqs = []

setup(name="oshino",
      version=get_version(),
      description="",
      author="zaibacu",
      packages=["oshino", "oshino.core", "oshino.agents"],
      install_requires=[str(ir.req) for ir in install_reqs],
      test_suite="pytest",
      tests_require=["pytest", "pytest-cov"],
      setup_requires=["pytest-runner"],
      entry_points={'console_scripts': ['oshino = oshino.run:main'
                                        ]
                    }
      )



import asyncio
import signal
import sys

from logbook import Logger, StreamHandler
from riemann_client.transport import TCPTransport
from riemann_client.client import QueuedClient
from functools import partial

from ..config import Config
from ..version import get_version
from . import send_heartbeat

loop = asyncio.get_event_loop()


def flush_riemann(client, transport, logger):
    try:
        transport.connect()
        client.flush()
        transport.disconnect()
    except ConnectionRefusedError as ce:
        logger.warn(ce)

async def main_loop(cfg: Config, logger: Logger):
    riemann = cfg.riemann
    transport = TCPTransport(riemann.host, riemann.port)
    client = QueuedClient(transport)
    agents = list(map(lambda x: (x.instance, x), cfg.agents))
    while True:
        send_heartbeat(client.event, logger)
        for agent, agent_cfg in agents:
            tags = [agent_cfg.tag] if agent_cfg.tag else None
            event_fn = partial(client.event,
                               tags=tags)
            await agent.process(event_fn)
        await asyncio.sleep(cfg.interval)
        flush_riemann(client, transport, logger)


def start_loop(cfg: Config):
    StreamHandler(sys.stdout).push_application()
    logger = Logger("Heart")
    logger.info("Initializing Oshino v{0}".format(get_version()))
    logger.info("Running forever in {0} seconds interval. Press Ctrl+C to exit"
                .format(cfg.interval))
    try:
        loop.run_until_complete(main_loop(cfg, logger))
    finally:
        loop.close()
